/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Xml;
//using Gtk;
using System.Collections.Generic;
using System.IO;

namespace Sagittarius.Config
{
	public class BookmarkManager
	{
		private List<string> bookmarkList;
		private string filename;
		
		public BookmarkManager ()
		{
			bookmarkList = new List<string>();
			filename = Misc.CONFIG_DIR+"/bookmarks";
			if(!Directory.Exists(Misc.CONFIG_DIR))
			{
				Directory.CreateDirectory(Misc.CONFIG_DIR);
			}
			ReadBookmarks();
		}
		
		public List<string> ReadBookmarks()
		{ 
			XmlDocument bookmarkXml = new XmlDocument(); // the documnet that will be readed
	 		try
			{
	 			bookmarkXml.Load(filename); // load the file
	 			XmlTextReader xr = new XmlTextReader(filename); // create a XmlTextREader in order to read file's content
				while(xr.Read())
				{  // start the main loop through the bookmark nodes of the file 
					if(xr.NodeType == XmlNodeType.Element) 
					{
						if(xr.Name=="bookmark")
						{ // if the element name is BOOKMARK then
							while(xr.MoveToNextAttribute())
							{ // read all the attributes and..
								if(xr.Name=="url") bookmarkList.Add(xr.Value); // ...add them to the arraylist
							}
						}
					}
				}
				xr.Close(); // close the XmlTextReader
			}
			catch(Exception)
			{ // this section will be used all the times the file is not present (first Cooka run)
				SaveBookmarks(true); // if there was an error, then the application save the bookmark creating the new setting file
			}
			return bookmarkList;
 		}
		
		public void SaveBookmarks(bool firsttime)
		{ 
			//Console.WriteLine("Entering BookmarkManager.SaveBookmarks");
			XmlDocument bookmarkXml = new XmlDocument(); 
			XmlElement xe = bookmarkXml.CreateElement("sagittarius-bookmarks"); // The root element of the xml settings file
			bookmarkXml.AppendChild(xe); // append the root element
			if(!firsttime) 
			{ // if this is not the first time, then add to the file all the available bookmarks (if present)
				foreach(string bookmark in bookmarkList)
				{
					XmlElement xpe = bookmarkXml.CreateElement("bookmark");
					xpe.SetAttribute("url",bookmark);
					xe.AppendChild(xpe);
				}
	 		}
			bookmarkXml.Save(filename);
			//Console.WriteLine("Leaving BookmarkManager.SaveBookmarks");
		}

		//public ListStore BookmarkList
		public List<string> BookmarkList
		{
			get { return bookmarkList; }
			set { bookmarkList = value; }
		}
		
		public void AddBookmark(string bookmarkString)
		{
			bookmarkList.Add(bookmarkString);
		}
		
		public void DeleteBookmark(string bookmarkToDelete)
		{
			bookmarkList.Remove(bookmarkToDelete);
		}
	}
}
