/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * The Sagittarius.ExiftoolWrapper.ExiftoolWrapper class is a C# port
 * of the ExiftoolWrapper java class originally created by Wyatt Olson.
 * The java ExiftoolWrapper class is part of the Moss project on
 * SourceForge.net <http://sourceforge.net/projects/moss/>.
 * The c# port is written by Robert C. Brinson, and is part of the
 * PictureTagger project on googleCode <http://picturetagger.googlecode.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PictureTagger.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;
using System.Diagnostics;
using Sagittarius.Config;


namespace Sagittarius.Exiftool
{
	public class ExiftoolProcess
	{
		private FileInfo exiftoolFile = null;
		private Process exiftoolProcess = null;
		
		public ExiftoolProcess(FileInfo exiftoolFile)
		{
			if (File.Exists(exiftoolFile.FullName) && (File.GetAttributes(exiftoolFile.FullName) & FileAttributes.Directory) == 0)
			{
				this.exiftoolFile = exiftoolFile;
			}
			else
			{
				throw new FileNotFoundException("The exiftool executable could not be found at the location supplied ("+exiftoolFile.Name+"). Please check that exiftool is installed and is located at the following path: " + exiftoolFile.FullName);
			}
		}
		
		private Process GenerateExiftoolProcess(string arguments)
		{
			Process exifTooProcess = null;
			ProcessStartInfo psi = new ProcessStartInfo();
			
			psi.FileName = exiftoolFile.FullName;
			if(Config.Preferences.CreateBackup)
			{
				psi.Arguments = " -m -n "+arguments;
			}
			else
			{
				psi.Arguments = " -m -n -overwrite_original "+arguments;
			}
			//Console.WriteLine(psi.Arguments);
			psi.RedirectStandardOutput = true;
			psi.UseShellExecute = false;
			
			try
			{
				exifTooProcess = Process.Start(psi);
			}
			catch (Exception e)
			{
				throw new ApplicationException("An error occurred while trying to execute exiftool with the following arguments: " + psi.Arguments, e);
			}

			return exifTooProcess;
		}
		
		public Stream GetBaseStreamFromExiftoolCommand(string command)
		{
			Stream exiftoolStream = null;
			try
			{
				exiftoolProcess = GenerateExiftoolProcess(command);
				exiftoolStream = exiftoolProcess.StandardOutput.BaseStream;
			}
			catch (Exception)
			{
				throw;
			}
			
			return exiftoolStream;
		}
		
		public string GetStreamAsStringFromExiftoolCommand(string command)
		{
			string exiftoolStream = String.Empty;
			//Console.WriteLine("the command is: "+command);
			try
			{
				exiftoolProcess = GenerateExiftoolProcess(command);
				exiftoolStream = exiftoolProcess.StandardOutput.ReadToEnd();
			}
			catch (Exception)
			{
				throw;
			}
			
			return exiftoolStream;
		}
		
		public void WaitForStreamExit()
		{
			if (exiftoolProcess != null)
			{
				exiftoolProcess.WaitForExit(1000);
			}
		}
	}
}