/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * The Sagittarius.ExiftoolWrapper.ExiftoolWrapper class is a C# port
 * of the ExiftoolWrapper java class originally created by Wyatt Olson.
 * The java ExiftoolWrapper class is part of the Moss project on
 * SourceForge.net <http://sourceforge.net/projects/moss/>.
 * The C# port is written by Robert C. Brinson, and is part of the
 * PictureTagger project on googleCode <http://picturetagger.googlecode.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PictureTagger.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;
using System.Text;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Sagittarius.Exiftool
{
	public class ExiftoolWrapper
	{
		private ExiftoolProcess exiftoolProcess;
		
		public ExiftoolWrapper (string exiftoolExecutable)
		{
			exiftoolProcess = new ExiftoolProcess(new FileInfo(exiftoolExecutable));
		}
		
		public Stream GetBinaryTagFromFile(string filename, string tagName)
		{
			//Console.WriteLine("Entering ExiftoolWrapper.GetBinaryTagFromFile");
			BufferedStream binTag;
			StringBuilder command = new StringBuilder();
			command.Append("-b");
			command.Append(" -" + tagName);
			command.Append(" ");
			//command.Append('"');
			command.Append(filename);
			//command.Append('"');
			try
			{
				binTag = new BufferedStream(exiftoolProcess.GetBaseStreamFromExiftoolCommand(command.ToString()));
				exiftoolProcess.WaitForStreamExit();
			}
			catch (Exception)
			{
				throw;
			}
			
			//Console.WriteLine("Leaving ExiftoolWrapper.GetBinaryTagFromFile");
			return binTag;
		}
		
		public string GetXmpTags(string fileName)
		{
			//Console.WriteLine("Executing ExiftoolWrapper.getXmpTags");
			return "\t"+GetStringFromExiftoolCommand(" -a -u -f -XMP:all "+"'"+fileName+"'");
		}
	
		public string WriteXmpTags(string filename, string args)
		{
			//Console.WriteLine("Executing ExiftoolWrapper.WriteXmpTags");
			//Console.WriteLine(filename+" "+ args);
			return GetStringFromExiftoolCommand( filename + " " + args );
		}
		
		public string CreateXmp (string filename)
		{
			//Console.WriteLine("Executing ExiftoolWrapper.CreateXmp");
			return GetStringFromExiftoolCommand(
			                                    " "+ 
			                                    '"'+
			                                    filename+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-exif:all>xmp:all"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:* --IPTC:Source --IPTC:Keywords > XMP:* "+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Source > XMP-photoshop:Source"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Keywords > XMP-dc:Subject"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:ObjectName > XMP-dc:Title"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:SpecialInstructions > XMP-photoshop:Instructions"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:By-line > XMP-dc:Creator"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:By-lineTitle > XMP-photoshop:AuthorsPosition"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Province-State > XMP-photoshop:State"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Country-PrimaryLocationName > XMP-photoshop:Country"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:OriginalTransmissionReference > XMP-photoshop:TransmissionReference"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:CopyrightNotice > XMP-dc:Rights"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Caption-Abstract > XMP-dc:Description"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Writer-Editor > XMP-photoshop:CaptionWriter"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Country-PrimaryLocationCode > XMP-iptcCore:CountryCode"+
			                                    '"'+
			                                    " "+
			                                    '"'+
			                                    "-IPTC:Sub-location > XMP-iptcCore:Location"+
			                                    '"');
		}
		
		public string SynchronizeToXmp (string filename)
		{
			//Console.WriteLine("Entering ExiftoolWrapper.SynchronizeToXmp");
			string output="";
			output += GetStringFromExiftoolCommand(
			                                       " "+
			                                       '"'+
			                                       filename+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP:* > IPTC:*"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-dc:Title > IPTC:ObjectName"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-dc:Subject > IPTC:Keywords"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-photoshop:Instructions > IPTC:SpecialInstructions"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-dc:Creator > IPTC:By-line"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-photoshop:AuthorsPosition > IPTC:By-lineTitle"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-photoshop:State > IPTC:Province-State"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-photoshop:Country > IPTC:Country-PrimaryLocationName"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-photoshop:TransmissionReference > IPTC:OriginalTransmissionReference"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-dc:Rights > IPTC:CopyrightNotice"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-dc:Description > IPTC:Caption-Abstract"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-photoshop:CaptionWriter > IPTC:Writer-Editor"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-iptcCore:CountryCode > IPTC:Country-PrimaryLocationCode"+
			                                       '"'+
			                                       " "+
			                                       '"'+
			                                       "-XMP-iptcCore:Location > IPTC:Sub-location"+
			                                       '"');
			//Console.WriteLine(output);
			//Console.WriteLine("Leaving ExiftoolWrapper.SynchronizeToXmp");
			return output;
			//# these tags are not free-form text, and may need special handling:
			//#-XMP-iptcCore:IntellectualGenre > IPTC:ObjectAttributeReference
			//#-XMP-iptcCore:SubjectCode > IPTC:SubjectReference
		}
		
		private string GetStringFromExiftoolCommand(string commandString)
		{
			//Console.WriteLine("Entering ExiftoolWrapper.GetStringFromExiftoolCommand");
			string output = "";
			try
			{
				output =exiftoolProcess.GetStreamAsStringFromExiftoolCommand(commandString);
				exiftoolProcess.WaitForStreamExit(); 
			}
			catch (Exception)
			{
				throw;
			}
			//Console.WriteLine("Leaving ExiftoolWrapper.GetStringFromExiftoolCommand");
			return output;
		}
	}
}