/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using Sagittarius.Exiftool;
using Sagittarius.Config;
using System.Collections.Generic;
using System.Text;

namespace Sagittarius.Metadata
{
	public static class MetadataParser
	{
		public static bool HasXmp(string filename)
		{
		return false;
		}
		
		public static XmpMetadata GetMetadata(string filename, out bool newXmp)
		{
			//Console.WriteLine("Entering Metadata.MetadatParser.GetMetadata. Filename: "+filename);
			newXmp=false;
			ExiftoolWrapper exiftoolWrapper = new ExiftoolWrapper(Config.Preferences.ExitoolExecutable);
			string metadataString = exiftoolWrapper.GetXmpTags(filename);
			string[] metadataLines = metadataString.Split(new char[] {'\n'});// split it to lines
			if(metadataLines.Length<=1)
			{
				newXmp=true;
				exiftoolWrapper.CreateXmp(filename);
				metadataString = exiftoolWrapper.GetXmpTags(filename);
				metadataLines = metadataString.Split(new char[] {'\n'});// split it to lines
			}
			exiftoolWrapper = null;
			//Console.WriteLine("Leaving Sagittarius.Metadata.MetadatParser.GetMetadata");
			return ParseXmp(metadataLines,filename);
		}

		public static XmpMetadata ParseXmp (string[] metadataLines,string filename)
		{
			//Console.WriteLine("Entering Sagittarius.Metadata.MetadatParser.ParseXmp");
			MetadataSection input = new MetadataSection();
			for (int i = 0; i<metadataLines.Length-1;i++) // put each line to a MetadataSection as a MetadataElement
			{
				string[] parts = metadataLines[i].Split(new string[] {": "},StringSplitOptions.None);
				input.AddTag(parts[0].Trim(),new MetadataTextTag(parts[1].Trim()));
				//Console.WriteLine(metadataLines[i]);
			}
			XmpMetadata xmp = new XmpMetadata();
			xmp.Filename=filename;
			// az új megközelítésben a szeksönökhöz is lehet adni elemeket
			// (momentán így kerülnek bele, szal nem is kell külön exif meg iptc szeksön
			// az egyes elementek maguk kezelik, hogy változtak-e, és a kezdeti értéküket tárolják.
			
			// Creator tag is not a simple text tag, but seqText
			// or, khm, currently bag-text, the sequential manner will be handled by the ui, i think
			xmp.IptcSect.AddTag(TagName.IPTC_CREATOR, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CREATOR)));
			xmp.IptcSect.AddTag(TagName.IPTC_CREATOR_JOB_TITLE, new MetadataTextTag (input.GetTagValueAsString(TagName.IPTC_CREATOR_JOB_TITLE)));
			// az address multiline, így a pontokkal tagolt részeket sortöréssel kell törni
			//xmp.IptcSect.AddTag(Tags.IPTC_CI_ADR_EXTADR, new MetadataTextTag(input.GetTagValueAsString(Tags.IPTC_CI_ADR_EXTADR).Replace('.','\n')));
			xmp.IptcSect.AddTag(TagName.IPTC_CI_ADR_EXTADR, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_ADR_EXTADR)));
			xmp.IptcSect.AddTag(TagName.IPTC_CI_ADR_CITY, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_ADR_CITY)));
			xmp.IptcSect.AddTag(TagName.IPTC_CI_ADR_REGION, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_ADR_REGION)));
			xmp.IptcSect.AddTag(TagName.IPTC_CI_ADR_PCODE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_ADR_PCODE)));
			xmp.IptcSect.AddTag(TagName.IPTC_CI_ADR_CTRY, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_ADR_CTRY)));
			// ez is multiline
			//xmp.IptcSect.AddTag(Tags.IPTC_CI_TEL_WORK, new MetadataTextTag(input.GetTagValueAsString(Tags.IPTC_CI_TEL_WORK).Replace('.','\n')));
			xmp.IptcSect.AddTag(TagName.IPTC_CI_TEL_WORK, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_TEL_WORK)));
			// ez is multiline, de itt a pontok szerint nem törhetek
			xmp.IptcSect.AddTag(TagName.IPTC_CI_EMAIL_WORK, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_EMAIL_WORK)));
			// ez is multiline, de itt a pontok szerint nem törhetek
			xmp.IptcSect.AddTag(TagName.IPTC_CI_URL_WORK, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CI_URL_WORK)));
			// az address multiline, így a pontokkal tagolt részeket sortöréssel kell törni
			//xmp.IptcSect.AddTag(Tags.IPTC_HEADLINE, new MetadataTextTag(input.GetTagValueAsString(Tags.IPTC_HEADLINE).Replace('.','\n')));
			xmp.IptcSect.AddTag(TagName.IPTC_HEADLINE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_HEADLINE)));
			// az address multiline, így a pontokkal tagolt részeket sortöréssel kell törni
			//xmp.IptcSect.AddTag(Tags.IPTC_DESCRIPTION, new MetadataTextTag (input.GetTagValueAsString(Tags.IPTC_DESCRIPTION).Replace('.','\n')));
			xmp.IptcSect.AddTag(TagName.IPTC_DESCRIPTION, new MetadataTextTag (input.GetTagValueAsString(TagName.IPTC_DESCRIPTION)));
			//Subject code tag is not a simple text tag, but bagText
			xmp.IptcSect.AddTag(TagName.IPTC_SUBJECT_CODE, new MetadataBagTextTag(input.GetTagValueAsString(TagName.IPTC_SUBJECT_CODE)));
			xmp.IptcSect.AddTag(TagName.IPTC_DESCRIPTION_WRITER, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_DESCRIPTION_WRITER)));
			//Keywords tag is not a simple text tag, but bagText
			xmp.IptcSect.AddTag(TagName.IPTC_KEYWORDS, new MetadataBagTextTag( input.GetTagValueAsString(TagName.IPTC_KEYWORDS)));
			//DateCreated tag is not a simple text tag, but Date
			xmp.IptcSect.AddTag(TagName.IPTC_DATE_CREATED, new MetadataDateTag(input.GetTagValueAsString(TagName.IPTC_DATE_CREATED)));
			xmp.IptcSect.AddTag(TagName.IPTC_INTELLECTUAL_GENRE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_INTELLECTUAL_GENRE)));
			//Scene tag is not a simple text tag, but bagText
			xmp.IptcSect.AddTag(TagName.IPTC_SCENE, new MetadataBagTextTag(input.GetTagValueAsString(TagName.IPTC_SCENE)));
			xmp.IptcSect.AddTag(TagName.IPTC_LOCATION, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_LOCATION)));
			xmp.IptcSect.AddTag(TagName.IPTC_CITY, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_CITY)));
			xmp.IptcSect.AddTag(TagName.IPTC_PROVINCE_STATE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_PROVINCE_STATE)));
			xmp.IptcSect.AddTag(TagName.IPTC_COUNTRY, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_COUNTRY)));
			xmp.IptcSect.AddTag(TagName.IPTC_COUNTRY_CODE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_COUNTRY_CODE)));
			xmp.IptcSect.AddTag(TagName.IPTC_TITLE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_TITLE)));
			xmp.IptcSect.AddTag(TagName.IPTC_JOB_ID, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_JOB_ID)));
			xmp.IptcSect.AddTag(TagName.IPTC_PROVIDER, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_PROVIDER)));
			// az address multiline, így a pontokkal tagolt részeket sortöréssel kell törni
			//xmp.IptcSect.AddTag(Tags.IPTC_INSTRUCTIONS, new MetadataTextTag(input.GetTagValueAsString(Tags.IPTC_INSTRUCTIONS).Replace('.','\n')));
			xmp.IptcSect.AddTag(TagName.IPTC_INSTRUCTIONS, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_INSTRUCTIONS)));
			xmp.IptcSect.AddTag(TagName.IPTC_SOURCE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_SOURCE)));
			xmp.IptcSect.AddTag(TagName.IPTC_COPYRIGHTED, new MetadataBoolTag(input.GetTagValueAsString(TagName.IPTC_COPYRIGHTED)));
			// az address multiline, így a pontokkal tagolt részeket sortöréssel kell törni
			//xmp.IptcSect.AddTag(Tags.IPTC_COPYRIGHT_NOTICE, new MetadataTextTag(input.GetTagValueAsString(Tags.IPTC_COPYRIGHT_NOTICE).Replace('.','\n')));
			xmp.IptcSect.AddTag(TagName.IPTC_COPYRIGHT_NOTICE, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_COPYRIGHT_NOTICE)));
			xmp.IptcSect.AddTag(TagName.IPTC_RIGHT_USAGE_TERMS, new MetadataTextTag(input.GetTagValueAsString(TagName.IPTC_RIGHT_USAGE_TERMS)));
			
			// exif szekció
			// átmeneti megoldásként az összes tag-et textként veszem fel
			xmp.ExifSect.AddTag(TagName.EXIF_MAKE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_MAKE )));
			xmp.ExifSect.AddTag(TagName.EXIF_MODEL, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_MODEL )));
			xmp.ExifSect.AddTag(TagName.EXIF_SERIAL, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SERIAL )));
			xmp.ExifSect.AddTag(TagName.EXIF_FIRMWARE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FIRMWARE )));
			xmp.ExifSect.AddTag(TagName.EXIF_SOFTWARE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SOFTWARE ))); 
			xmp.ExifSect.AddTag(TagName.EXIF_LENS, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_LENS )));
			xmp.ExifSect.AddTag(TagName.EXIF_MAX_APERTURE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_MAX_APERTURE )));
			xmp.ExifSect.AddTag(TagName.EXIF_DATETIME_ORIGINAL, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_DATETIME_ORIGINAL )));
			xmp.ExifSect.AddTag(TagName.EXIF_DATETIME_DIGITIZED, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_DATETIME_DIGITIZED )));
			xmp.ExifSect.AddTag(TagName.EXIF_EXPOSURE_TIME, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_EXPOSURE_TIME )));
			xmp.ExifSect.AddTag(TagName.EXIF_SHUTTER_SPEED, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SHUTTER_SPEED )));
			xmp.ExifSect.AddTag(TagName.EXIF_APERTURE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_APERTURE )));
			xmp.ExifSect.AddTag(TagName.EXIF_ISO, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_ISO )));
			xmp.ExifSect.AddTag(TagName.EXIF_EXPOSURE_COMP, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_EXPOSURE_COMP )));
			xmp.ExifSect.AddTag(TagName.EXIF_EXPOSURE_PROG, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_EXPOSURE_PROG )));
			xmp.ExifSect.AddTag(TagName.EXIF_EXPOSURE_MODE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_EXPOSURE_MODE )));
			xmp.ExifSect.AddTag(TagName.EXIF_METERING_MODE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_METERING_MODE )));
			xmp.ExifSect.AddTag(TagName.EXIF_SCENE_CAPTURE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SCENE_CAPTURE )));
			xmp.ExifSect.AddTag(TagName.EXIF_FOCAL_LENGTH, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FOCAL_LENGTH )));
			xmp.ExifSect.AddTag(TagName.EXIF_FOCAL_LENGTH_35, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FOCAL_LENGTH_35 )));
			xmp.ExifSect.AddTag(TagName.EXIF_DIGITAL_ZOOM, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_DIGITAL_ZOOM )));
			xmp.ExifSect.AddTag(TagName.EXIF_FNUMBER, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FNUMBER )));
			xmp.ExifSect.AddTag(TagName.EXIF_LIGHT_SOURCE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_LIGHT_SOURCE )));
			xmp.ExifSect.AddTag(TagName.EXIF_WHITE_BALANCE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_WHITE_BALANCE )));
			xmp.ExifSect.AddTag(TagName.EXIF_FLASH_FIRED, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FLASH_FIRED )));
			xmp.ExifSect.AddTag(TagName.EXIF_FLASH_MODE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FLASH_MODE )));
			xmp.ExifSect.AddTag(TagName.EXIF_FLASH_REDEYE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FLASH_REDEYE )));
			xmp.ExifSect.AddTag(TagName.EXIF_FLASH_RETURN, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FLASH_RETURN )));
			xmp.ExifSect.AddTag(TagName.EXIF_FLASH_FUNCTION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FLASH_FUNCTION )));
			xmp.ExifSect.AddTag(TagName.EXIF_FLASH_ENERGY, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_FLASH_ENERGY )));
			xmp.ExifSect.AddTag(TagName.EXIF_IMAGE_WIDTH, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_IMAGE_WIDTH )));
			xmp.ExifSect.AddTag(TagName.EXIF_IMAGE_LENGTH, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_IMAGE_LENGTH )));
			xmp.ExifSect.AddTag(TagName.EXIF_ORIENTATION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_ORIENTATION )));
			xmp.ExifSect.AddTag(TagName.EXIF_BITS_PER_SAMPLE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_BITS_PER_SAMPLE )));
			xmp.ExifSect.AddTag(TagName.EXIF_COMPRESSION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_COMPRESSION )));
			xmp.ExifSect.AddTag(TagName.EXIF_COLOR_SPACE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_COLOR_SPACE )));
			xmp.ExifSect.AddTag(TagName.EXIF_RESOLUTION_X, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_RESOLUTION_X )));
			xmp.ExifSect.AddTag(TagName.EXIF_RESOLUTION_Y, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_RESOLUTION_Y )));
			xmp.ExifSect.AddTag(TagName.EXIF_RESOLUTION_UNIT, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_RESOLUTION_UNIT )));
			xmp.ExifSect.AddTag(TagName.EXIF_BRIGHTNESS, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_BRIGHTNESS )));
			xmp.ExifSect.AddTag(TagName.EXIF_CONTRAST, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_CONTRAST )));
			xmp.ExifSect.AddTag(TagName.EXIF_SATURATION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SATURATION )));
			xmp.ExifSect.AddTag(TagName.EXIF_SHARPNESS, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SHARPNESS )));
			xmp.ExifSect.AddTag(TagName.EXIF_GAIN_CONTROL, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GAIN_CONTROL )));
			xmp.ExifSect.AddTag(TagName.EXIF_CUSTOM_RENDERED, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_CUSTOM_RENDERED )));
			xmp.ExifSect.AddTag(TagName.EXIF_SUBJECT_DISTANCE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SUBJECT_DISTANCE )));
			xmp.ExifSect.AddTag(TagName.EXIF_SUBJECT_LOCATION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SUBJECT_LOCATION )));
			xmp.ExifSect.AddTag(TagName.EXIF_SUBJECT_AREA, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SUBJECT_AREA )));
			xmp.ExifSect.AddTag(TagName.EXIF_SUBJECT_DISTANCE_RANGE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_SUBJECT_DISTANCE_RANGE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_VERSION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_VERSION )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_LATITUDE_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_LATITUDE_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_LATITUDE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_LATITUDE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_LONGITUDE_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_LONGITUDE_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_LONGITUDE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_LONGITUDE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_ALTITUDE_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_ALTITUDE_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_ALTITUDE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_ALTITUDE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DATETIME, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DATETIME )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_SATELLITES, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_SATELLITES )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_STATUS, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_STATUS )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_MEASURE_MODE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_MEASURE_MODE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DOP, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DOP )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_SPEED_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_SPEED_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_SPEED, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_SPEED )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_TRACK_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_TRACK_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_TRACK, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_TRACK )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_IMG_DIRECTION_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_IMG_DIRECTION_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_IMG_DIRECTION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_IMG_DIRECTION )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_MAP_DATUM, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_MAP_DATUM )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DEST_LATITUDE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DEST_LATITUDE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DEST_LONGITUDE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DEST_LONGITUDE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DEST_BEARING_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DEST_BEARING_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DEST_BEARING, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DEST_BEARING )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DEST_DISTANCE_REF, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DEST_DISTANCE_REF )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DEST_DISTANCE, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DEST_DISTANCE )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_PROCESSING_METHOD, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_PROCESSING_METHOD )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_AREA_INFORMATION, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_AREA_INFORMATION )));
			xmp.ExifSect.AddTag(TagName.EXIF_GPS_DIFFERENTIAL, new MetadataTextTag(input.GetTagValueAsString(TagName.EXIF_GPS_DIFFERENTIAL )));
			return xmp;
			
		}

		public static void WriteMetadata(XmpMetadata metadata, string filename)
		{
			//Console.WriteLine("Entering Sagittarius.Metadata.MetadataParser.WriteMetadata");
			StringBuilder sb = new StringBuilder();
			MetadataBagTextTag tempTag;
			string tempTagName;
			
			//creator. It is a MetadataTextElement, although it should be seq....
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CREATOR))
			{
				sb.Append(" -XMP:" + TagName.IPTC_CREATOR + "=" + "'" +metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CREATOR) + "'");
			}
			//creator job title
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CREATOR_JOB_TITLE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_CREATOR_JOB_TITLE.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CREATOR_JOB_TITLE) + "'");
			}
			// contactinfo members have different tagnames for setting and displaying values. ExiftoolTagnames -fields can't be used for the corresponding tags, they would not be set (although they are good for reading)
			//creator ContactInfo Address
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_ADR_EXTADR))
			{
				sb.Append(" -XMP:CreatorContactInfoCiAdrExtadr=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_EXTADR) + "'");
			}
			//creator ContactInfo City
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_ADR_CITY))
			{
				sb.Append(" -XMP:CreatorContactInfoCiAdrCity=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_CITY) + "'");
			}
			// creator Contactinfo Region
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_ADR_REGION))
			{
				sb.Append(" -XMP:CreatorContactInfoCiAdrRegion=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_REGION) + "'");
			}
			// creator Contactinfo Postal Code
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_ADR_PCODE))
			{
				sb.Append(" -XMP:CreatorContactInfoCiAdrPcode=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_PCODE) + "'");
			}
			// creator Contactinfo Country 
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_ADR_CTRY))
			{
				sb.Append(" -XMP:CreatorContactInfoCiAdrCtry=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_ADR_CTRY) + "'");
			}
			// creator Contactinfo Te"lephone 
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_TEL_WORK))
			{
				sb.Append(" -XMP:CreatorContactInfoCiTelWork=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_TEL_WORK) + "'");
			}
			// creator Contactinfo email 
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_EMAIL_WORK))
			{
				sb.Append(" -XMP:CreatorContactInfoCiEmailWork=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_EMAIL_WORK) + "'");
			}
			// creator Contactinfo web url 
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CI_URL_WORK))
			{
				sb.Append(" -XMP:CreatorContactInfoCiUrlWork=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CI_URL_WORK) + "'");
			}
			// headline
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_HEADLINE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_HEADLINE + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_HEADLINE) + "'");
			}
			// description
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_DESCRIPTION))
			{
				sb.Append(" -XMP:" + TagName.IPTC_DESCRIPTION + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_DESCRIPTION) + "'");
			}
			// subject code
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_SUBJECT_CODE))
			{
				tempTag = metadata.IptcSect.GetTag(TagName.IPTC_SUBJECT_CODE) as Sagittarius.Metadata.MetadataBagTextTag;
				tempTagName = TagName.IPTC_SUBJECT_CODE;
				foreach(string removedBagElement in tempTag.GetRemovedBagElements())
				{
					sb.Append(" -XMP:" + tempTagName + "-=" + "'"+ removedBagElement + "'");
				}
				foreach(string addedBagElement in tempTag.GetAddedBagElements())
				{
					sb.Append(" -XMP:" + tempTagName + "+=" + "'"+ addedBagElement + "'");
				}
			}
			// description writer
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_DESCRIPTION_WRITER))
			{
				sb.Append(" -XMP:" + TagName.IPTC_DESCRIPTION_WRITER.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_DESCRIPTION_WRITER) + "'");
			}
			// keywords
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_KEYWORDS))
			{
				tempTag = metadata.IptcSect.GetTag(TagName.IPTC_KEYWORDS) as Sagittarius.Metadata.MetadataBagTextTag;
				tempTagName = TagName.IPTC_KEYWORDS;
				foreach(string removedBagElement in tempTag.GetRemovedBagElements())
				{
					sb.Append(" -XMP:" + tempTagName + "-=" + "'"+ removedBagElement + "'");
				}
				foreach(string addedBagElement in tempTag.GetAddedBagElements())
				{
					sb.Append(" -XMP:" + tempTagName + "+=" + "'"+ addedBagElement + "'");
				}
			}
			// iptc date created
			// it is of MetadataDateElement type
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_DATE_CREATED))
			{
				sb.Append(" -XMP:" + TagName.IPTC_DATE_CREATED.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_DATE_CREATED) + "'");
			}
			// intellectual genre
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_INTELLECTUAL_GENRE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_INTELLECTUAL_GENRE.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_INTELLECTUAL_GENRE) + "'");
			}
			// scene
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_SCENE))
			{
				tempTag = metadata.IptcSect.GetTag(TagName.IPTC_SCENE) as Sagittarius.Metadata.MetadataBagTextTag;
				tempTagName = TagName.IPTC_SCENE;
				foreach(string removedBagElement in tempTag.GetRemovedBagElements())
				{
					sb.Append(" -XMP:" + tempTagName + "-=" + "'"+ removedBagElement + "'");
				}
				foreach(string addedBagElement in tempTag.GetAddedBagElements())
				{
					sb.Append(" -XMP:" + tempTagName + "+=" + "'"+ addedBagElement + "'");
				}
			}
			// location
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_LOCATION))
			{
				sb.Append(" -XMP:" + TagName.IPTC_LOCATION + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_LOCATION) + "'");
			}
			// city
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_CITY))
			{
				sb.Append(" -XMP:" + TagName.IPTC_CITY + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_CITY) + "'");
			}
			// state
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_PROVINCE_STATE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_PROVINCE_STATE + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_PROVINCE_STATE) + "'");
			}
			// country
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_COUNTRY))
			{
				sb.Append(" -XMP:" + TagName.IPTC_COUNTRY + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COUNTRY) + "'");
			}
			// country code
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_COUNTRY_CODE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_COUNTRY_CODE.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COUNTRY_CODE) + "'");
			}
			// title
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_TITLE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_TITLE + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_TITLE) + "'");
			}
			// jobId
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_JOB_ID))
			{
				sb.Append(" -XMP:" + TagName.IPTC_JOB_ID.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_JOB_ID) + "'");
			}
			// provider
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_PROVIDER))
			{
				sb.Append(" -XMP:" + TagName.IPTC_PROVIDER + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_PROVIDER) + "'");
			}
			// instructions
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_INSTRUCTIONS))
			{
				sb.Append(" -XMP:" + TagName.IPTC_INSTRUCTIONS + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_INSTRUCTIONS) + "'");
			}
			// source
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_SOURCE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_SOURCE + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_SOURCE) + "'");
			}
			// Marked, MetadataBoolElement type
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_COPYRIGHTED))
			{
				sb.Append(" -XMP:" + TagName.IPTC_COPYRIGHTED + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COPYRIGHTED) + "'");
			}
			// copyright notice
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_COPYRIGHT_NOTICE))
			{
				sb.Append(" -XMP:" + TagName.IPTC_COPYRIGHT_NOTICE.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_COPYRIGHT_NOTICE) + "'");
			}
			// right usage terms
			if(metadata.IptcSect.GetTagChanged(TagName.IPTC_RIGHT_USAGE_TERMS))
			{
				sb.Append(" -XMP:" + TagName.IPTC_RIGHT_USAGE_TERMS.Replace(" ", "") + "=" + "'" + metadata.IptcSect.GetTagValueAsString(TagName.IPTC_RIGHT_USAGE_TERMS) + "'");
			}

			ExiftoolWrapper eW = new ExiftoolWrapper(Config.Preferences.ExitoolExecutable);
			string args = sb.ToString();
			if(sb.Length>0)
			{
				eW.WriteXmpTags(filename, args);
				eW.SynchronizeToXmp(filename);
			}
			//Console.WriteLine("Leaving Sagittarius.Metadata.MetadataParser.WriteMetadata");
		}
	}
}
