/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;
using System.Collections.Generic;
using System.Text;

namespace Sagittarius.Metadata
{
	public class MetadataSection
	{
		protected Dictionary<string,IMetadataTag> tagList;
		private string sectionName;

#region konstruktorok

		public MetadataSection ()
		{
			//Console.WriteLine("Entering Metadata.MetadataSection constructor()");
			this.tagList = new Dictionary<string,IMetadataTag>();
			//Console.WriteLine("Leaving Metadata.MetadataSection constructor()");
		}

		public MetadataSection (string sectionName)
		{
			//Console.WriteLine("Entering Metadata.MetadataSection constructor(string sectionName). SectionName: "+sectionName);
			this.sectionName = sectionName;
			this.tagList = new Dictionary<string, IMetadataTag>();
			//Console.WriteLine("Leaving Metadata.MetadataSection constructor(string sectionName)");
		}
		
		public MetadataSection (string sectionName, Dictionary<string,IMetadataTag> tagList)
		{
			//Console.WriteLine("Entering Metadata.MetadataSection constructor(string sectionName). SectionName: "+sectionName);
			this.sectionName = sectionName;
			this.tagList = tagList;
			//Console.WriteLine("Leaving Metadata.MetadataSection constructor(string sectionName)");
		}
#endregion

#region property metódusok

		public string SectionName 
		{
			get { return sectionName; }
			set { sectionName = value; }
		}
		
#endregion

#region element access methods

		public virtual void AddTag (string key, IMetadataTag newTag)
		{
			try
			{
				tagList.Add(key, newTag);
			}
			catch(System.ArgumentException)
			{
			}
		}
		
		public IMetadataTag GetTag(string key)
		{
			try
			{
				return tagList[key];
			}
			catch(System.Collections.Generic.KeyNotFoundException)
			{
				return null;
			}
		}

		public virtual object GetTagValue(string key)
		{
			try
			{
				return tagList[key].TagValue;
			}
			catch(System.Collections.Generic.KeyNotFoundException)
			{
				return null;
			}
		}
		
		public virtual string GetTagValueAsString (string key)
		{
			//Console.WriteLine("Entering Metadata.MetadataSection.GetElementAsString. Key: "+key);
			string result = "";
			try
			{
				result =  tagList[key].GetValueAsString();
			}
			catch(System.Collections.Generic.KeyNotFoundException)
			{
				result = "";
			}
			//Console.WriteLine("Leaving Metadata.MetadataSection.GetElementAsString");
			return result;
		}
		
		public virtual void SetTagValue (string key, object newValue)
		{
			if(newValue!=null)
			{ 
				tagList[key].TagValue=newValue; 
			}
			else
			{
				AddTag(key,newValue as IMetadataTag);
			}
		}
		
		public virtual void SetTag (string key, IMetadataTag newValue)
		{
			if(newValue!=null)
			{ tagList[key]=newValue; }
		}
		
		public virtual void SetTagValueFromString(string key, string newValue)
		{
			//Console.WriteLine("Entering Metadata.MetadataSection.SetElementValueFromString. Key: "+key+" Value: "+newValue);
			tagList[key].SetValueFromString(newValue);
		}
		
		public virtual Dictionary<string, IMetadataTag> TagsDict
		{
			get { return tagList; }
		}
		
		public void SaveTagChanges(string key)
		{
			tagList[key].SaveChanges();
		}
		
		public void DiscardTagChanges(string key)
		{
			tagList[key].DiscardChanges();
		}
		
		public bool GetTagChanged(string key)
		{
			return tagList[key].ValueChanged;
		}
			
#endregion

		public override string ToString ()
		{
			StringBuilder result = new StringBuilder();
			foreach(KeyValuePair<string, IMetadataTag> tag in tagList)
			{
				result.Append(tag.Key+": "+tag.Value.GetValueAsString()+"\n");
			}
			return this.sectionName+"\n"+result.ToString();
		}
		
		public void Add(MetadataSection sectionToAdd)
		{
			foreach(KeyValuePair<string,IMetadataTag> tagToAdd in sectionToAdd.TagsDict)
			{
				if(this.GetTag(tagToAdd.Key)!=null)
				{
					this.GetTag(tagToAdd.Key).Add(tagToAdd.Value);
				}
				else
				{
					this.AddTag(tagToAdd.Key,tagToAdd.Value.Copy());
				}
			}
		}
	}
}
