/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;


namespace Sagittarius.Metadata
{
	public class MetadataBagTextTag: IMetadataTag
	{
		private List<string> mValue;
		private List<string> originalValue;
		private bool changed;
		// theese two is for Exiftool interoperability.
		// the methods associated with them too
		private List<string> addedElements;
		private List<string> removedElements;
		
		public MetadataBagTextTag (string list)
		{
			mValue = new List<string>();
			string[] parts = list.Split(',',';');
			foreach(string part in parts)
			{
				mValue.Add(part.Trim());
			}
			Init();
		}
		
		public MetadataBagTextTag(List<string> bag)
		{
			this.mValue = bag;
			Init();
		}
		
		private void Init()
		{
			//Console.WriteLine("Entering MetadataBagTextTag.Init");
			originalValue = new List<string>(mValue);
			addedElements = new List<string>();
			removedElements = new List<string>();
			SetChangedStatus();
			//Console.WriteLine("Leaving MetadataBagTextTag.Init");
		}
		
		public string GetBagElement(int index)
		{
			//Console.WriteLine("Entering Metadata.MetadataBagTextElement.GetBagElement");
			return mValue[index];
			//Console.WriteLine("Leaving Metadata.MetadataBagTextElement.RemoveBagElement");
		}
		
		public void AddBagElement(string newElement)
		{
			mValue.Add(newElement);
			addedElements.Add(newElement);
			SetChangedStatus();
		}
		
		public void RemoveBagElement(string element)
		{
			removedElements.Add(element);
			mValue.Remove(element);
			SetChangedStatus();
		}
		
		public void RemoveBagElement (int index)
		{
			//Console.WriteLine("Entering Metadata.MetadataBagTextElement.RemoveBagElement");
			removedElements.Add(mValue[index]);
			mValue.RemoveAt(index);
			SetChangedStatus();
			//Console.WriteLine("Leaving Metadata.MetadataBagTextElement.RemoveBagElement");
		}
		
		private void Housekeeping()
		{
			removedElements.Clear();
			addedElements.Clear();
			// végignézzük az original listát. Ha van olyan eleme, mely nincs meg az újban, akkor az removelistre kerül
			foreach(string piece in originalValue)
			{
				if(!mValue.Contains(piece))
				{
					removedElements.Add(piece);
				}
			}
			// végignézzük az új listát. Ha van olyan eleme, mely nincs meg az originalban, akkor az addlistre kerül
			foreach(string piece in mValue)
			{
				if(!originalValue.Contains(piece))
				{
					addedElements.Add(piece);
				}
			}
			/*if((addedElements.Count>0)||(removedElements.Count>0))
			{
				changed=true;
			}*/
			SetChangedStatus();
		}
		
		public object TagValue
		{
			get { return mValue;}
			set 
			{
				mValue=(List<string>)value;
				Housekeeping();
			}
		}
		
		public void SetValueFromString (string newValue)
		{
			//Console.WriteLine("Entering Metadata.MetadataBagTextTag.SetValueFromString");
			// lehetséges, hogy több elem volt szelektálva, és egy olyan
			// baglistet kapunk, amiben vannak *-os elemek
			// ezek esetében meg kell vizsgálni, hogy az eredeti lista tartalmazta-e őket
			// ha igen, akkor benne kell legyenek
			List<string> commons = new List<string>();
			List<string> noncommons = new List<string>();
			string[] pieces = newValue.Split(',',';');
			// szétválasztom a lista tagjait commonokra és noncommonokra
			// de előtte leveszem a kezdő és záró space-karaktereket
			for (int i=0;i<pieces.Length;i++)
			{
				pieces[i]=pieces[i].Trim();
			}
			foreach (string piece in pieces)
			{
				//Console.WriteLine("  megnézzük a "+piece+" darabot");
				if(piece.StartsWith("*"))
				{
					//Console.WriteLine("   csillaggal kezdődik, hozzáadjuk a nonkommonokhoz");
					noncommons.Add(piece);
				}
				else
				{
					//Console.WriteLine("   nem kezdődik *-al, hozzáadjuk a kommonokhoz");
					commons.Add(piece);
				}
			}
			mValue = new List<string>();
			// a commonokat mind hozzáadjuk az új listához
			//Console.WriteLine(" a kommonokat mind hozzáadjuk a mValue-hoz");
			foreach (string piece in commons)
			{
				mValue.Add(piece);
			}
			// megnézzük a kapott lista commonjait
			// ha egy noncommon szerepel a régi listán, azt hozzá kel adni az újhoz
			// de előbb meg kell őket szaadítani a csillagtól
			for(int i=0; i<noncommons.Count;i++)
			{
				noncommons[i]=noncommons[i].Remove(0,1);
			}
			foreach(string piece in noncommons)
			{
				//Console.WriteLine("  megnézzük a "+piece+" nonkommon darabot");
				if(originalValue.Contains(piece))
				{
					//Console.WriteLine("   originalValue tartalmazza, hozzáadjuk az mValue-hoz ");
					mValue.Add(piece);
				}
				//else
				//{
					//Console.WriteLine("   originalValue nem tartalmazza, kidobjuk ");
				//}
			}
			Housekeeping();
		}
		
		public string GetValueAsString ()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder(); 
			foreach (string piece in mValue)
			{
				sb.Append(piece+", ");
			}
			if(sb.Length>2)
			{
				sb.Remove(sb.Length-2,2);
			}
			return sb.ToString();
		}
		
		public string[] GetAddedBagElements()
		{
			return addedElements.ToArray();
		}
		
		public string[] GetRemovedBagElements()
		{
			return removedElements.ToArray();
		}

		public bool ValueChanged 
		{
			get { return changed; }
		}
		
		public void DiscardChanges ()
		{
			mValue = new List<string>(originalValue);
			addedElements.Clear();
			removedElements.Clear();
			SetChangedStatus();
		}
		
		public void SaveChanges ()
		{
			originalValue = mValue;
			addedElements.Clear();
			removedElements.Clear();
			SetChangedStatus();
		}
		
		public override string ToString ()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			foreach(string str in mValue)
			{
				sb.Append(str +", ");
			}
			sb.Remove(sb.Length-2,2);
			return sb.ToString();
		}
		
		public void Add(IMetadataTag tagToAdd)
		{
			//Console.WriteLine("Entering Metadata.MetadataBagTextTag.Add");
			//Console.WriteLine(" taking tag:" +tagToAdd.GetValueAsString());
			if(tagToAdd is MetadataBagTextTag)
			{
				//Console.WriteLine("  tag is good type..");
				List<string> piecesToAdd = tagToAdd.TagValue as List<string>;
				List<string> commons = new List<string>();
				List<string> noncommons = new List<string>();
				// vegyük az adott tag által tartalmazott keywordoket
				// mivel a kommonoknak minden tagban ott kell lenni
				// ha ezeken végigiterálok, és megnézem, hogy megvannak-e a többiben, és megvannak, akkor kommon
				// de lehet,hogy ez már egy akkumulált lista, és vannak köztük *-al kezdődőek. azokat ki kell hagyni
				if(mValue.Count>0)
				{
					//Console.WriteLine("   original pieces number: "+mValue.Count);
					foreach(string originalPiece in mValue)
					{
						if ((!originalPiece.StartsWith("*"))&&(piecesToAdd.Contains(originalPiece)))
						{
							commons.Add(originalPiece);
						}
					}
				}
				// nemkommon lista létrehozása
				// végigiterálunk minden lista minden keywordjén
				// ha az adott keyword nem szerepel egyelten listán sem, akkor 
				// hozzáadjuk a noncommonokhoz
				// kezdünk a tag saját listájával
				//Console.WriteLine("  nonkommlista...");
				foreach(string piece in mValue)
				{
					//Console.WriteLine("   vizsgált elem: "+piece);
					if(commons.Contains(piece))
					{
						continue;
						//Console.WriteLine("   a kommon listán van");
					}
					else if ((noncommons.Contains(piece))||(noncommons.Contains("*"+piece)))
					{
						continue;
						//Console.WriteLine("   a nonkommon listán van");
					}
					else
					{
						//Console.WriteLine("   egyik listán sincs, hozzáadom a nonkommonhoz");
						noncommons.Add(piece);
					}
				}
				// majd jön a hozzáadandó lista
				//Console.WriteLine("  nonkommlista...");
				foreach(string piece in piecesToAdd)
				{
					//Console.WriteLine("   vizsgált elem: "+piece);
					if(commons.Contains(piece))
					{
						continue;
						//Console.WriteLine("   a kommon listán van");
					}
					else if ((noncommons.Contains(piece))||(noncommons.Contains("*"+piece)))
					{
						continue;
						//Console.WriteLine("   a nonkommon listán van");
					}
					else
					{
						//Console.WriteLine("   egyik listán sincs, hozzáadom a nonkommonhoz");
						noncommons.Add(piece);
					}
				}
				// a noncommonokat csillaggal kell kezdeni
				for(int i =0; i<noncommons.Count ;i++)
				{
					if(!noncommons[i].StartsWith("*"))
					{
						noncommons[i] =noncommons[i].Insert(0,"*");
					}
				}
				// az egészet hozzá kell adni az mValue-hoz
				mValue.Clear();
				mValue.AddRange(commons);
				mValue.AddRange(noncommons);
			}
			else
			{
				throw new Exception("Type mismatch");
			}
			SetChangedStatus();
		}
		
		public IMetadataTag Copy()
		{
			return new Sagittarius.Metadata.MetadataBagTextTag(this.GetValueAsString());
		}

		private void SetChangedStatus()
		{
			//Console.WriteLine("Entering MetadataBagTextTag.SetChangedStatus");
			changed = false;
			foreach(string piece in originalValue)
			{
				if(!mValue.Contains(piece))
				{
					changed = true;
				}
			}
			foreach(string piece in mValue)
			{
				if(!originalValue.Contains(piece))
				{
					changed = true;
				}
			}
			//Console.WriteLine("Leaving MetadataBagTextTag.SetChangedStatus. Changed is: "+changed.ToString());
		}

	}
}
