/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;

namespace Sagittarius.Metadata
{
	public class MetadataBoolTag:IMetadataTag
	{
		private MBool mValue; 
		private MBool originalValue;
		private bool changed;
		
		public MetadataBoolTag(MBool mValue)
		{
			//Console.WriteLine("Entering Metadata.MetadataBoolElement constructor");
			this.mValue = mValue;
			originalValue = mValue;
			SetChangedStatus();
			//Console.WriteLine("Leaving Metadata.MetadataBoolElement constructor");
		}
		
		public MetadataBoolTag (string valueAsString)
		{
			//Console.WriteLine("Entering Metadata.MetadataBoolElement constructor");
			SetValueFromString(valueAsString);
			originalValue = mValue;
			SetChangedStatus();
			//Console.WriteLine("Leaving Metadata.MetadataBoolElement constructor");
		}
		
		public string GetValueAsString()
		{
			//Console.WriteLine("Entering Metadata.MetadataTextElement.GetValueAsString");
			string result="";
			if (mValue==MBool.False)
			{
				result = "False";
			}
			else if (mValue == MBool.True)
			{
				result = "True";
			}
			return result;
			//Console.WriteLine("Leaving Metadata.MetadataTextElement.GetValueAsString");
		}
		
		public void SetValueFromString (string newValue)
		{
			if (newValue=="True")
			{
				mValue = MBool.True;
			}
			else if (newValue =="False")
			{
				mValue = MBool.False;
			}
			else
			{
				mValue = MBool.Unknown;
			}
			SetChangedStatus();
		}
		
		public object TagValue 
		{
			get { return mValue; }
			set 
			{
				//Console.WriteLine("Entering Metadata.MetadataBoolTag.TagValue");
				mValue = (MBool)value;
				SetChangedStatus();
				//Console.WriteLine("Leaving Metadata.MetadataBoolTag.TagValue");
			}
		}
		
		public override string ToString ()
		{
			return mValue.ToString();
		}

		public bool ValueChanged 
		{
			get { return changed; }
		}
		
		public void DiscardChanges ()
		{
			mValue = originalValue;
			SetChangedStatus();
		}
		
		public void SaveChanges ()
		{
			originalValue = mValue;
			SetChangedStatus();
		}

		public void Add(IMetadataTag tagToAdd)
		{
			if(tagToAdd is MetadataBoolTag)
			{
				if ((MBool)tagToAdd.TagValue!=(MBool)mValue)
				{
					mValue = MBool.Unknown;
				}
			}
			else
			{
				throw new Exception("Type mismatch");
			}
			SetChangedStatus();
		}

		public IMetadataTag Copy()
		{
			return new MetadataBoolTag((MBool)this.TagValue);
		}

		private void SetChangedStatus()
		{
			//Console.WriteLine("Entering Metadata.MetadataBoolTag.SetChangedStatus");
			//Console.WriteLine("original: "+originalValue.ToString()+ " mValue: "+mValue.ToString());
			if(mValue==originalValue)
			{
				changed = false;
			}
			else
			{
				changed = true;
			}
			//Console.WriteLine("Leaving Metadata.MetadataBoolTag.SetChangedStatus");
		}
		
	}
}
