/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;

namespace Sagittarius.Metadata
{
	public class MetadataDateTag : IMetadataTag 
	{
		private MDate mValue;
		private MDate originalValue;
		private bool changed;
		
		public MetadataDateTag(MDate mDate)
		{
			this.mValue = mDate;
			originalValue = mValue;
			SetChangedStatus();
		}
		
		public MetadataDateTag(string dateString)
		{
			mValue = MDate.SetFromString(dateString);
			originalValue = mValue;
			SetChangedStatus();
		}
		
		public object TagValue 
		{
			get { return mValue; }
			set 
			{
				mValue = (MDate)value;
				SetChangedStatus();
			}
		}
		
		public string GetValueAsString ()
		{
			return MDate.GetAsString(mValue);
		}
		
		public void SetValueFromString (string newValue)
		{
			mValue = MDate.SetFromString(newValue);
			SetChangedStatus();
		}

		public bool ValueChanged 
		{
			get { return changed; }
		}
		
		public void DiscardChanges ()
		{
			mValue = originalValue;
			SetChangedStatus();
		}
		
		public void SaveChanges ()
		{
			originalValue = mValue;
			SetChangedStatus();
		}
		
		public override string ToString ()
		{
			return base.ToString ();
		}

		public void Add( IMetadataTag tagToAdd)
		{
			if(tagToAdd is MetadataDateTag)
			{
				if ((MDate)tagToAdd.TagValue !=mValue)
				{
					mValue = new MDate(new DateTime (),0);
				}
			}
			else
			{
				throw new Exception("Type mismatch");
			}
			SetChangedStatus();
		}

		public IMetadataTag Copy()
		{
			return new MetadataDateTag((MDate)this.TagValue);
		}

		private void SetChangedStatus()
		{
			if(mValue==originalValue)
			{
				changed = false;
			}
			else
			{
				changed = true;
			}
		}
		
	}
}
