/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;

namespace Sagittarius.Metadata
{
	public class MetadataTextTag:IMetadataTag
	{
		private string mValue;
		private string originalValue; 
		private bool changed;
		
		public MetadataTextTag(string mValue)
		{
			//Console.WriteLine("Entering Metadata.MetadataTextElement constructor");
			this.originalValue=mValue;
			this.mValue = mValue;
			SetChangedStatus();
			//Console.WriteLine("Leaving Metadata.MetadataTextElement constructor");
		}
		
		public string GetValueAsString()
		{
			//Console.WriteLine("Entering Metadata.MetadataTextElement.GetValueAsString");
			//Console.WriteLine("Leaving Metadata.MetadataTextElement.GetValueAsString");
			return mValue;
		}
		
		public void SetValueFromString (string newValue)
		{
			//Console.WriteLine("Entering Metadata.MetadataTextTag.SetValueFromString");
			mValue = newValue;
			SetChangedStatus();
			//Console.WriteLine("Leaving Metadata.MetadataTextTag.SetValueFromString");
		}
		
		public object TagValue 
		{
			get { return mValue; }
			set
			{
				mValue = (string)value;
				SetChangedStatus();
			}
		}
		
		public bool ValueChanged 
		{
			get 
			{
				return changed; 
			}
		}
		
		public void DiscardChanges ()
		{
			mValue = originalValue;
			SetChangedStatus();
		}
		
		public void SaveChanges ()
		{
			//Console.WriteLine("Entering Metadata.MetadataTextTag.SaveChanges");
			originalValue = mValue;
			SetChangedStatus();
			//Console.WriteLine("Leaving Metadata.MetadataTextTag.SaveChanges");
		}
		
		public override string ToString ()
		{
			return mValue;
		}
		
		public void Add(IMetadataTag tagToAdd)
		{
			if(tagToAdd is MetadataTextTag)
			{
				if (tagToAdd.GetValueAsString()!=mValue)
				{
					mValue = Strs.MULTIPLE_VALUES;
				}
			}
			else
			{
				throw new Exception("Type mismatch");
			}
			SetChangedStatus();
		}
		
		public IMetadataTag Copy()
		{
			return new MetadataTextTag(this.TagValue as string);
		}
		
		private void SetChangedStatus()
		{
			if(mValue==originalValue)
			{
				changed = false;
			}
			else
			{
				changed = true;
			}
		}
		
	}
}
