/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;

namespace Sagittarius.Metadata
{
	public struct MDate
	{
		private DateTime date;
		private int timeZone;
		
		public MDate (DateTime date, int timeZone)
		{
			this.date = date;
			this.timeZone=timeZone;
		}
		
		public DateTime Date 
		{
			get { return date;}
			set { date = value; }
		}
		
		public int TimeZone
		{
			get { return timeZone; }
			set
			{
				if ((value<-12)||(value>12)) 
				{
					throw new ArgumentOutOfRangeException("TimeZone","The given Time zone value is out of range (-12 - 12)"); 
				}
				else 
				{
					timeZone = value;
				}
			}
		}
		
		public void Set (DateTime date, int timeZone)
		{
			this.date=date;
			this.timeZone = timeZone;
		}
		
		public MDate Get ()
		{
			return this;
		}
		
		public override string ToString ()
		{
			return date.ToString()+" "+timeZone.ToString();
		}

		#region static

		public static bool operator == (MDate lhs, MDate rhs)
		{
			bool result;
			if((lhs.Date==rhs.Date)&&(lhs.TimeZone==rhs.TimeZone))
			{
				result = true;
			}
			else
			{
				result =false;
			}
			return result;
		}

		public static bool operator != (MDate lhs, MDate rhs)
		{
			bool result;
			if((lhs.Date==rhs.Date)&&(lhs.TimeZone==rhs.TimeZone))
			{
				result = false;
			}
			else
			{
				result =true;
			}
			return result;
		}
		
		public override int GetHashCode ()
		{
			return base.GetHashCode ();
		}
		
		public override bool Equals (object o)
		{
			return base.Equals (o);
		}
 

		
		public static MDate SetFromString(string inputStr)
		{
			//Console.WriteLine("Entering Metadata.MDate.SetFromString. input: "+inputStr);
			DateTime dateTime;
			int timeZone=0;
			switch(inputStr.Length)
			{
			case 10:
			{
				dateTime=new DateTime(
					Convert.ToInt32(inputStr.Substring(0,4)),
					Convert.ToInt32(inputStr.Substring(5,2)),
					Convert.ToInt32(inputStr.Substring(8,2)));
				break;
			}
			case 20:
			{
				dateTime=new DateTime(
					Convert.ToInt32(inputStr.Substring(0,4)),
					Convert.ToInt32(inputStr.Substring(5,2)),
					Convert.ToInt32(inputStr.Substring(8,2)),
					Convert.ToInt32(inputStr.Substring(11,2)),
					Convert.ToInt32(inputStr.Substring(14,2)),
					Convert.ToInt32(inputStr.Substring(17,2)));
				break;
			}
			case 27:
			{
				dateTime=new DateTime(
					Convert.ToInt32(inputStr.Substring(0,4)),
					Convert.ToInt32(inputStr.Substring(5,2)),
					Convert.ToInt32(inputStr.Substring(8,2)),
					Convert.ToInt32(inputStr.Substring(11,2)),
					Convert.ToInt32(inputStr.Substring(14,2)),
					Convert.ToInt32(inputStr.Substring(17,2)));
				break;
			}
				
			default:
			{
				dateTime = new DateTime();
				break;
			}
			}
			if(inputStr.Length>19)
			{
				if(inputStr.Substring(19,1)=="Z")
				{
					timeZone=0;
				}
				else
				{
					timeZone=Convert.ToInt32(inputStr.Substring(20,3));
				}
			}
			else
			{
				timeZone=0;
			}
			//Console.WriteLine(" result: "+dateTime.ToString() + timeZone.ToString() );
			//Console.WriteLine("Leaving Metadata.MDate.SetFromString");
			return new MDate(dateTime, timeZone);
		}
		
		public static string GetAsString(MDate mDate)
		{
			string tZ;
			if (mDate.TimeZone<0)
			{
				tZ= mDate.TimeZone.ToString()+":000";
			}
			else
			{
				tZ= "+"+mDate.TimeZone.ToString()+":000";
			}
			if(tZ.Length==6)
			{
				tZ=tZ.Insert(1,"0");
			}
			string year = mDate.Date.Year.ToString();
			string month = mDate.Date.Month.ToString();
			if(month.Length==1)
			{
				month="0"+month;
			}
			string day = mDate.Date.Day.ToString();
			if(day.Length==1)
			{
				day="0"+day;
			}
			string hour = mDate.Date.Hour.ToString();
			if(hour.Length==1)
			{
				hour="0"+hour;
			}
			string minute = mDate.Date.Minute.ToString();
			if(minute.Length==1)
			{
				minute="0"+minute;
			}
			string second = mDate.Date.Second.ToString();
			if(second.Length==1)
			{
				second="0"+second;
			}
			return year + ":" + month + ":" + day + " " + hour + ":" + minute + ":" + second + "." + tZ;
		}
#endregion		
	}
}
