/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.IO;
using Sagittarius.Exiftool;

namespace Sagittarius.Metadata
{
	public class XmpMetadata
	{
		private MetadataSection iptcSect, exifSect;
		private string filename;
		private bool newXmp=false;
		private bool changed;

#region konstruktorok

		public XmpMetadata(string filename)
		{
			//Console.WriteLine("Entering Sagittarius.Metadata.XmpMetadata constructor");
			this.filename=filename;
			Init();
			//Console.WriteLine("Leaving Sagittarius.Metadata.XmpMetadata constructor");
		}
		
		public XmpMetadata()
		{
			//Console.WriteLine("Entering Sagittarius.Metadata.XmpMetadata constructor");
			Init();
			//Console.WriteLine("Leaving Sagittarius.Metadata.XmpMetadata constructor");
		}
		
		private void Init()
		{
			exifSect = new MetadataSection("EXIF");
			iptcSect = new MetadataSection("IPTC");
			changed = false;
			newXmp=false;
		}

#endregion

#region property
		
		public MetadataSection IptcSect
		{
			get { return iptcSect; }
			set { iptcSect=value; }
		}
		
		public string Filename
		{
			get { return filename; }
			set { filename = value; }
		}

		public MetadataSection ExifSect
		{
			get { return exifSect; }
			set { exifSect = value; }
		}
		
		public bool NewXmp
		{
			get { return newXmp; }
			set { newXmp=value; }
		}
		
		public bool Changed
		{
			get
			{
				if(!changed)
				{
					foreach(KeyValuePair<string, IMetadataTag> element in iptcSect.TagsDict)
					{
						if(element.Value.ValueChanged)
						{
							changed=true;
						}
					}
					foreach(KeyValuePair<string, IMetadataTag> element in exifSect.TagsDict)
					{
						if(element.Value.ValueChanged)
						{
							changed=true;
						}
					}
				}
				return changed;
			}
		}
		
#endregion

#region public methods
		
		public void DiscardChanges()
		{
			foreach(KeyValuePair<string, IMetadataTag> element in iptcSect.TagsDict)
			{
				element.Value.DiscardChanges();
			}
			foreach(KeyValuePair<string, IMetadataTag> element in exifSect.TagsDict)
			{
				element.Value.DiscardChanges();
			}
			changed = false;
		}
		
		public void SaveChanges()
		{
			foreach(KeyValuePair<string, IMetadataTag> element in iptcSect.TagsDict)
			{
				element.Value.SaveChanges();
			}
			foreach(KeyValuePair<string, IMetadataTag> element in exifSect.TagsDict)
			{
				element.Value.SaveChanges();
			}
			changed = false;
		}
		
		public override string ToString()
		{
			return "XMP metadata abstraction for the file: "+ filename+"\r\n"+exifSect.ToString()+ "\r\n"+ iptcSect.ToString();
		}
		
		public void Add(XmpMetadata metadataToAdd)
		{
			this.iptcSect.Add(metadataToAdd.IptcSect);
			this.exifSect.Add(metadataToAdd.ExifSect);
			this.Filename="multiple files";
		}
		
#endregion
	}
}


