/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;
using Gnome;

namespace Sagittarius
{
	/// <summary>
	/// Class for containing a System.IO.DirectoryInfo class, and a few additional elements and methods
	/// </summary>
	public class DirInfo
	{
		private DirectoryInfo directoryInfo;
		private Gdk.Pixbuf icon;
		private string name, fullName;
		private bool explored, explorable;
		public bool hasSubDirectories=false;
		/// <summary>
		/// Normal constructor for directories
		/// </summary>
		/// <param name="dirName">
		/// A string containing the full path of the directory<see cref="System.String"/>
		/// </param>
		public DirInfo(string dirName /*,bool isHome*/)
		{
			icon = Gtk.IconTheme.Default.LoadIcon("gnome-fs-directory",16,Gtk.IconLookupFlags.ForceSvg); // get the folder icon
			directoryInfo = new DirectoryInfo(dirName);
			name = directoryInfo.Name;
			fullName=directoryInfo.FullName;
			try
			{
				if (directoryInfo.GetDirectories().Length>0)
				{
					hasSubDirectories=true;
				}
			}
			catch
			{}
			//Console.WriteLine("Exiting DirInfo constructor");
		}
		/// <summary>
		/// Constructor for non-existing directories
		/// This constructor is called when a dummy DirInfo object is needed
		/// to be placed under an existing DirInfo object on the TreeStore,
		/// for the TreeView to display expanders before the directory name
		/// </summary>
		public DirInfo()
		{
			//Console.WriteLine("Entering DirInfo constructor");
			//Console.WriteLine("argument: (empty)");
			name = "   (empty)";
			fullName="";
			//Console.WriteLine("Exiting DirInfo constructor");
		}
		
		public Gdk.Pixbuf Icon
		{
			get { return icon; }
		}
		
		public string Name
		{
			get { return name; }
		}
		public string FullName
		{
			get { return fullName; }
		}
		public bool Explored
		{
			get { return explored; }
		}
		public bool Explorable
		{
			get
			{
				try
				{
					directoryInfo.GetDirectories();
					explorable=true;
				}
				catch
				{
					explorable=false;
				}
				return explorable;
			}
		}
		/// <summary>
		/// Method for extracting the subdirectores of a directory
		/// </summary>
		/// <returns>
		/// A DirInfo[] containing the subdurectories<see cref="DirInfo"/>
		/// </returns>
		public DirInfo[] GetDirectories()
		{
			//Console.WriteLine("Entering DirInfo.GetDirectories");
			DirectoryInfo[] directories = directoryInfo.GetDirectories();
			DirInfo[] children = new DirInfo[directories.Length];
			for (int i = 0; i<children.Length;i++)
			{
				children[i]= new DirInfo(directories[i].FullName);
			}
			explored=true;
			//Console.WriteLine("Exiting Dirinfo.GetDirectories");
			return children;
		}
	}
}
