/* Copyright 2008 Daniel Vokó <sagittarius@vokod.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using Gtk;
using Sagittarius.Metadata;

namespace Sagittarius.Widgets
{
	
	
	public partial class DateEdit : Gtk.Bin
	{
		
#region fields
		private MDate currentDate;
#endregion
		
#region constructor
		
		public DateEdit()
		{
			this.Build();
			CurrentDateTime = new MDate(new DateTime(),0);
		}
		
#endregion
		
#region property
		
		public MDate CurrentDateTime
		{
			get
			{
				return currentDate;
			}
			set
			{
				currentDate = value;
				dateEdit.CurrentDate=value.Date;
				timeZoneSb.Value=value.TimeZone;
			}
		}
	
		public bool IsValid
		{
			get { return dateEdit.IsDateValid(); }
		}
		
#endregion
		
#region code

		public override string ToString()
		{
			//Console.WriteLine("Entering Sagittarius.Widgets.HollyDatePicker.ToString.");
			string timeZone;
			if (timeZoneSb.Value>0)
			{
				timeZone = "+"+timeZoneSb.Value.ToString();
			}
			else
			{
				timeZone = timeZoneSb.Value.ToString();
			}
			string result = dateEdit.CurrentDate.ToString()+" "+timeZone;
			//Console.WriteLine("ToString: "+result);
			//Console.WriteLine("Entering Sagittarius.Widgets.HollyDatePicker.ToString.");
			return result;
		}

#endregion
		
#region eventhandlers

		protected virtual void OnDateEditDateChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering Widgets.DateEdit.OnDateEditDateChanged");
			if(IsValid)
			{
				currentDate.Date=dateEdit.CurrentDate;
			}
			RaiseChangedEvent();
			//Console.WriteLine("Leaving Widgets.DateEdit.OnDateEditDateChanged");
		}

		protected virtual void OnTimeZoneSbValueChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering Widgets.DateEdit.OnTimeZoneSbValueChanged");
			currentDate.TimeZone=(int)timeZoneSb.Value;
			RaiseChangedEvent();
			//Console.WriteLine("Leaving Widgets.DateEdit.OnTimeZoneSbValueChanged");
		}
		
		public event ChangedHandler Changed ;
		
		public delegate void ChangedHandler (object sender, System.EventArgs args);
		
		private void RaiseChangedEvent ()
		{
			//Console.WriteLine("Entering Widgets.DateEdit.RaiseChangedEvent");
			if (Changed!=null)
			{
				Changed(this,new System.EventArgs());
			}
			//Console.WriteLine("Leaving Widgets.DateEdit.RaiseChangedEvent");
		}
		
#endregion
	}
}
